function getFromPropertyTree(path, default_value, create) {
  /*
    This function retrieves a variable from the property tree. If
    the variable does not exist and the flag create is true, a new
    variable is created with value default_value.

    returns the read variable or undefined
  */

  var myvar = Property.getProperty(path);

  if ((myvar === null || myvar === undefined)) {
    if (create) {
      myvar = default_value;
      Property.setProperty(path, myvar);
      Property.setFlag(path, 'ARCHIVE', true); //set ARCHIVE flag to active
      Property.store();

    } else {
      return undefined; // variable does not exist and will not be created
    }
  }

  return myvar;
}

function getFromPropertyTree_nostore(path, default_value, create) {
  /*
    This function retrieves a variable from the property tree. If
    the variable does not exist and the flag create is true, a new
    variable is created with value default_value. This function does
    not store the property tree, it has to be done after the function is called.

    returns the read variable or undefined
  */

  var myvar = Property.getProperty(path);

  if ((myvar === null || myvar === undefined)) {
    if (create) {
      myvar = default_value;
      Property.setProperty(path, myvar);
      Property.setFlag(path, 'ARCHIVE', true); //set ARCHIVE flag to active

    } else {
      return undefined; // variable does not exist and will not be created
    }
  }

  return myvar;
}


function setArchiveFlag(path) {
  Property.setFlag(path, 'ARCHIVE', true); //set ARCHIVE flag to active
  Property.store();
}

function saveToPropertyTree(path, var_value, create) {
  /*
    This function stores a value to the property tree.

    path is the variable's  relative path

    create is a boolean. When true,  the variable is created if it did not exist before.
    When false,  the variable is only stored if it existed previously.

  */

  var myvar = Property.getProperty(path);

  if (myvar === null || myvar === undefined) {
    if (create) {
      Property.setProperty(path, var_value);
      Property.setFlag(path, 'ARCHIVE', true); //set flag to active
      Property.store();
    }
  } else {
    Property.setProperty(path, var_value);
    Property.store();
  }
}

function saveToPropertyTree_nostore(path, var_value, create) {
  /*
    This function stores a value to the property tree but does not
    store the property tree.

    path is the variable's  relative path

    create is a boolean. When true,  the variable is created if it did not exist before.
    When false,  the variable is only stored if it existed previously.

  */

  var myvar = Property.getProperty(path);

  if (myvar === null || myvar === undefined) {
    if (create) {
      Property.setProperty(path, var_value);
      Property.setFlag(path, 'ARCHIVE', true); //set flag to active
    }
  } else {
    Property.setProperty(path, var_value);
  }
}


function initializePropertyTree() {
  /*
    This function retrieves all the required variables from the
    property tree.  Non-existent variables are created and assigned
    a default value.
  */


  //Notifications
  var Devices = [];


  getFromPropertyTree_nostore("Notifications/PushTargets", JSON.stringify(Devices), true);
  getFromPropertyTree_nostore("Notifications/PushTargetOptions", JSON.stringify(Devices), true);
  getFromPropertyTree_nostore("Notifications/EmailAddress", DEFAULT_STRING, true);


  // Controller
  //getFromPropertyTree_nostore("Controller/ClusterID",16,true);
  saveToPropertyTree_nostore("Controller/ClusterID", 19, true); // has to be created dynamically
  getFromPropertyTree_nostore("Controller/CurrentLevel", 3, true); //nominal

  // Exhaust flap
  getFromPropertyTree_nostore("ExhaustFlap", DEFAULT_STRING, true);
  //saveToPropertyTree_nostore("ExhaustFlap",DEFAULT_STRING,true);


  //Airboxes FilterReplacementInterval in [day]
  getFromPropertyTree_nostore("Airboxes/FilterReplacementInterval", FilterReplacementInterval, true); //365
  //saveToPropertyTree_nostore("Airboxes/FilterReplacementInterval",3,true); //365

  var dateReplacement = new Date();
  getFromPropertyTree_nostore("Airboxes/LastFilterReplacementDate", dateReplacement.toISOString(), true);

  //Prevent that app be uninstalled (????)
  saveToPropertyTree_nostore("system/preventUninstall", false, true);

  Property.store();

  debug("DEBUG", "Loaded property tree");
}